/**********************************************************************
Copyright (c) 2009 Andy Jefferson and others. All rights reserved. 
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.store.mapped;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Definition of the mapping of result expressions in a datastore statement.
 * In a typical RDBMS SQL statement the result clause will contain many expressions.
 * If the overall result of the statement maps to a class then we use StatementMappingForClass.
 * If the overall result is simply various expressions then we use this.
 */
public class StatementResultMapping
{
    /** Mappings for the result expressions keyed by the position. */
    Map<Integer, StatementMappingIndex> mappings = null;

    public StatementResultMapping()
    {
    }

    /**
     * Accessor for the mapping information for the result expression at the position.
     * @param position The position in the result
     * @return The mapping information
     */
    public StatementMappingIndex getMappingForResultExpression(int position)
    {
        if (mappings == null)
        {
            return null;
        }
        return mappings.get(position);
    }

    public void addMappingForResultExpression(int position, StatementMappingIndex mapping)
    {
        if (mappings == null)
        {
            mappings = new HashMap<Integer, StatementMappingIndex>();
        }
        mappings.put(position, mapping);
    }

    public boolean isEmpty()
    {
        return (mappings == null || mappings.size() == 0);
    }

    public int getNumberOfResultExpressions()
    {
        return (mappings != null ? mappings.size() : 0);
    }

    public String toString()
    {
        StringBuffer str = new StringBuffer("StatementResults:");
        if (mappings != null)
        {
            Iterator<Map.Entry<Integer, StatementMappingIndex>> mapIter = mappings.entrySet().iterator();
            while (mapIter.hasNext())
            {
                Map.Entry<Integer, StatementMappingIndex> entry = mapIter.next();
                str.append(" position=").append(entry.getKey());
                str.append(" mapping=").append(entry.getValue());
                if (mapIter.hasNext())
                {
                    str.append(",");
                }
            }
        }
        return str.toString();
    }
}