/**********************************************************************
 Copyright (c) 2007 Andy Jefferson and others. All rights reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Contributors:
 ...
 **********************************************************************/
package org.datanucleus.store.mapped.scostore;

import java.util.Collection;
import java.util.List;
import java.util.ListIterator;

import org.datanucleus.ManagedConnection;
import org.datanucleus.ObjectManager;
import org.datanucleus.StateManager;
import org.datanucleus.store.mapped.exceptions.MappedDatastoreException;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.query.ResultObjectFactory;

/**
 * Specialization interface for {@link AbstractListStore}.
 * {@link AbstractListStore} delegates to instances of this interface for behavior
 * that is tied to a specific datastore implementation.
 */
public interface AbstractListStoreSpecialization extends AbstractCollectionStoreSpecialization
{
    int indexOf(StateManager sm, Object element, ElementContainerStore ecs);

    int lastIndexOf(StateManager sm, Object element, ElementContainerStore ecs);

    int[] getIndicesOf(StateManager sm, Collection elements, ElementContainerStore ecs);

    ListIterator listIterator(QueryExpression stmt, boolean useUpdateLock, ObjectManager om,
                              StateManager ownerSM, ResultObjectFactory rof, AbstractListStore als);

    List internalGetRange(ObjectManager om, boolean useUpdateLock, QueryExpression stmt, ResultObjectFactory getROF,
                          ElementContainerStore ecs);

    int[] internalShift(StateManager ownerSM, ManagedConnection conn, boolean batched, int oldIndex, int amount,
                        boolean executeNow, ElementContainerStore ecs) throws MappedDatastoreException;
}
