/**********************************************************************
Copyright (c) 2007 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/

package org.datanucleus.store.mapped.scostore;

import org.datanucleus.StateManager;
import org.datanucleus.Transaction;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;
import org.datanucleus.store.query.ResultObjectFactory;

/**
 * Specialization interface for {@link AbstractMapStore}. {@link AbstractMapStore} delegates to instances of this interface for
 * behavior that is tied to a specific datastore implementation.
 */
public interface AbstractMapStoreSpecialization extends ElementContainerStoreSpecialization
{
    /**
     * @see AbstractMapStore#updateEmbeddedValue
     */
    boolean updateEmbeddedValue(StateManager sm, Object value, int fieldNumber, Object newValue, JavaTypeMapping fieldMapping,
                                AbstractMapStore mapStore);

    /**
     * @see AbstractMapStore#updateEmbeddedKey
     */
    boolean updatedEmbeddedKey(StateManager sm, Object key, int fieldNumber, Object newValue, JavaTypeMapping fieldMapping,
                               AbstractMapStore mapStore);

    /**
     * @see AbstractMapStore#containsValue
     */
    boolean containsValue(StateManager sm, Object value, AbstractMapStore abstractMapStore);

    Object getValue(StateManager sm, Object key, QueryExpression stmt, Transaction tx, boolean useUpdateLock, AbstractMapStore mapStore,
                    ResultObjectFactory rof);

    void initialise(AbstractMapStore mapStore);
}
