/**********************************************************************
Copyright (c) 2007 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.store.mapped.scostore;

import org.datanucleus.ClassLoaderResolver;
import org.datanucleus.StateManager;
import org.datanucleus.store.mapped.DatastoreContainerObject;
import org.datanucleus.store.mapped.DatastoreIdentifier;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;
import org.datanucleus.store.mapped.query.DiscriminatorIteratorStatement;
import org.datanucleus.store.mapped.query.UnionIteratorStatement;

/**
 * Specialization interface for {@link ElementContainerStore}.
 * {@link ElementContainerStore} delegates to instances of this interface for behavior
 * that is tied to a specific datastore implementation.
 */
public interface ElementContainerStoreSpecialization
{
    void executeClear(StateManager ownerSM, ElementContainerStore elementContainerStore);

    int getSize(StateManager sm, ElementContainerStore elementContainerStore);

    DiscriminatorIteratorStatement newDiscriminatorIteratorStatement(ClassLoaderResolver clr, Class[] cls, boolean includeSubclasses,
                                                                     boolean selectDiscriminator);

    DiscriminatorIteratorStatement newDiscriminatorIteratorStatement(ClassLoaderResolver clr, Class[] cls, boolean b,
                                                                     boolean b1, boolean allowsNull,
                                                                     DatastoreContainerObject containerTable,
                                                                     JavaTypeMapping elementMapping, DatastoreIdentifier elmIdentifier);

    UnionIteratorStatement newUnionIteratorStatement(ClassLoaderResolver clr, Class candidateType,
                                                     boolean includeSubclasses, Class sourceType,
                                                     JavaTypeMapping sourceMapping, DatastoreContainerObject sourceTable,
                                                     boolean sourceJoin, Boolean withMetadata, boolean joinToExcludeTargetSubclasses,
                                                     boolean allowsNull);
}
